@@ -63,6 +63,11 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
 /*--*/
 
 /* System library. */
@@ -114,6 +119,7 @@ static const BOUNCE_TEMPLATE def_bounce_failure_template = {
     "us-ascii",
     MAIL_ATTR_ENC_7BIT,
     MAIL_ADDR_MAIL_DAEMON " (Mail Delivery System)",
+    "Mail Delivery System <" MAIL_ADDR_MAIL_DAEMON ">",
     "Undelivered Mail Returned to Sender",
     "Postmaster Copy: Undelivered Mail",
     def_bounce_failure_body,
@@ -130,8 +136,7 @@ static const char *def_bounce_delay_body[] = {
     "# THIS IS A WARNING ONLY.  YOU DO NOT NEED TO RESEND YOUR MESSAGE. #",
     "####################################################################",
     "",
-    "Your message could not be delivered for more than $delay_warning_time_hours hour(s)."
-    ,
+    "Your message could not be delivered for more than $delay_warning_time_hours hour(s).",
     "It will be retried until it is $maximal_queue_lifetime_days day(s) old.",
     "",
     "For further assistance, please send mail to " MAIL_ADDR_POSTMASTER ".",
@@ -150,6 +155,7 @@ static const BOUNCE_TEMPLATE def_bounce_delay_template = {
     "us-ascii",
     MAIL_ATTR_ENC_7BIT,
     MAIL_ADDR_MAIL_DAEMON " (Mail Delivery System)",
+    "Mail Delivery System <" MAIL_ADDR_MAIL_DAEMON ">",
     "Delayed Mail (still being retried)",
     "Postmaster Warning: Delayed Mail",
     def_bounce_delay_body,
@@ -179,6 +185,7 @@ static const BOUNCE_TEMPLATE def_bounce_success_template = {
     "us-ascii",
     MAIL_ATTR_ENC_7BIT,
     MAIL_ADDR_MAIL_DAEMON " (Mail Delivery System)",
+    "Mail Delivery System <" MAIL_ADDR_MAIL_DAEMON ">",
     "Successful Mail Delivery Report",
     0,
     def_bounce_success_body,
@@ -205,6 +212,7 @@ static const BOUNCE_TEMPLATE def_bounce_verify_template = {
     "us-ascii",
     MAIL_ATTR_ENC_7BIT,
     MAIL_ADDR_MAIL_DAEMON " (Mail Delivery System)",
+    "Mail Delivery System <" MAIL_ADDR_MAIL_DAEMON ">",
     "Mail Delivery Status Report",
     0,
     def_bounce_verify_body,
@@ -238,7 +246,7 @@ void    bounce_templates_free(BOUNCE_TEMPLATES *bs)
     bounce_template_free(bs->delay);
     bounce_template_free(bs->success);
     bounce_template_free(bs->verify);
-    myfree((char *) bs);
+    myfree((void *) bs);
 }
 
 /* bounce_templates_load - load template or group from stream */
@@ -299,7 +307,7 @@ void    bounce_templates_load(VSTREAM *fp, BOUNCE_TEMPLATES *ts)
 	    }
 	    if (vstream_feof(fp))
 		msg_warn("%s, line %d: missing \"%s\" end marker",
-			  VSTREAM_PATH(fp), lineno, value);
+			 VSTREAM_PATH(fp), lineno, value);
 	    member_name = STR(saved_member_name);
 	    value = STR(multi_line_buf);
 	}
